#! /usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const swagger_parser_1 = __importDefault(require("@apidevtools/swagger-parser"));
const quicktype_core_1 = require("quicktype-core");
const nunjucks_1 = __importDefault(require("nunjucks"));
const filter_1 = require("./filter");
const fs_1 = require("fs");
const path_1 = require("path");
const yaml_1 = __importDefault(require("yaml"));
const addon_1 = require("./addon");
let inputData;
const schema_mapping = [];
const schemas_cache = {};
async function schemas(language) {
    if (schemas_cache[language]) {
        return schemas_cache[language];
    }
    const res = await (0, quicktype_core_1.quicktype)({
        inputData,
        lang: "python",
        rendererOptions: {
            "just-types": "false",
            "nice-property-names": "true",
        },
    });
    schemas_cache[language] = res.lines.join("\n");
    return schemas_cache[language];
}
async function type(language, type_name) {
    const s = await schemas(language);
    const re = new RegExp(`^def (.*)_from_dict\\(.*\\) -> (.*):$`, "gm");
    const m = Array.from(s.matchAll(re));
    const i = schema_mapping.indexOf(type_name);
    if (i >= 0)
        return m[i][2];
    return undefined;
}
async function transformerFromDict(language, type_name) {
    const s = await schemas(language);
    const re = new RegExp(`^def (.*)_from_dict\\(.*\\) -> (.*):$`, "gm");
    const m = Array.from(s.matchAll(re));
    const i = schema_mapping.indexOf(type_name);
    if (i >= 0)
        return m[i][1] + "_from_dict";
    return undefined;
}
async function transformerToDict(language, type_name) {
    const s = await schemas(language);
    const re = new RegExp(`^def (.*)_from_dict\\(.*\\) -> (.*):$`, "gm");
    const m = Array.from(s.matchAll(re));
    const i = schema_mapping.indexOf(type_name);
    if (i >= 0)
        return m[i][1] + "_to_dict";
    return undefined;
}
async function main() {
    commander_1.program
        .name("openapi-codegeneration")
        .description("Generate code from an OpenAPI service")
        .version("0.2.0")
        .requiredOption("-i, --input <strings...>", "openapi input file")
        .requiredOption("-t, --template <string>", "openapi template name")
        .requiredOption("-o, --output <string>", "openapi output directory")
        .option("-f, --filters <string...>", "filter collections to load")
        .option("--keep-refs", "keep references")
        .addOption(new commander_1.Option("-p, --preset <string>")
        .implies({ template: "" })
        .conflicts("-f"));
    commander_1.program.parse();
    const options = commander_1.program.opts();
    if (options.filters) {
        options.filters.foreach((fc) => (0, addon_1.activateFilterCollection)(fc, env));
    }
    let templateDir = (0, path_1.resolve)(__dirname, "../templates", options.template);
    let preset;
    if (options.preset) {
        preset = await (0, addon_1.loadPreset)(options.preset);
        templateDir = preset.templatesDir;
    }
    if (options.template.startsWith(".")) {
        templateDir = (0, path_1.resolve)(process.cwd(), options.template);
    }
    const env = nunjucks_1.default.configure(templateDir, {
        autoescape: false,
        noCache: true,
    });
    env.addFilter("formatPath", filter_1.formatPath_filter);
    env.addFilter("hasPathParameter", filter_1.hasPathParameter_filter);
    env.addFilter("upperCamelCase", filter_1.upperCamelCase_filter);
    env.addFilter("lowerCamelCase", filter_1.lowerCamelCase_filter);
    env.addFilter("lowerSnakeCase", filter_1.lowerSnakeCase_filter);
    env.addFilter("replaceRegEx", filter_1.replaceRegEx_filter);
    env.addFilter("jsf", filter_1.jsf_filter);
    env.addFilter("permuteOptionalArgs", filter_1.permuteOptionalArgs_filter);
    env.addFilter("await", filter_1.await_filter, true);
    env.addGlobal("schemas", schemas);
    env.addGlobal("type", type);
    env.addGlobal("transformerFromDict", transformerFromDict);
    env.addGlobal("transformerToDict", transformerToDict);
    if (preset) {
        (0, addon_1.activatePreset)(preset, env);
    }
    const inputs = options.input;
    function loadAndDeref(input) {
        const file = (0, fs_1.readFileSync)(input, "utf8");
        const schema = yaml_1.default.parse(file);
        return JSON.stringify(schema).replace(/{[^{}]*"\$ref":"([^"]*)"[^{}]*}/g, (_substring, group) => {
            return loadAndDeref((0, path_1.resolve)((0, path_1.join)(input, "..", group)));
        });
    }
    inputData = new quicktype_core_1.InputData();
    let api = undefined;
    for (const input of inputs) {
        try {
            const openApi = (options["keepRefs"]
                ? await swagger_parser_1.default.parse(input)
                : await swagger_parser_1.default.validate(input));
            if (openApi.openapi !== "3.1.0") {
                console.error(`Only OpenAPI 3.1.0 is supported, but ${openApi.openapi} was provided. Please upgrade your OpenAPI file.`);
                process.exit(1);
            }
            if (api !== undefined) {
                console.error("Multiple OpenAPI Specifications in input files. This tools only support one OpenAPI document at a time.");
                process.exit(1);
            }
            api = openApi;
            inputData = extractSchemasFromOpenAPI(inputData, api);
        }
        catch (e) {
            // parse input as normal yaml file
            const schema = JSON.parse(loadAndDeref(input));
            addJsonSchema(schema.title ?? input, schema, inputData);
        }
    }
    const context = {
        api,
        ...api,
        schemas,
    };
    env.addFilter("resolveRef", (0, filter_1.resolveRef_meta_filter)(api));
    env.addFilter("jsonSchemaCombineAllOf", (0, filter_1.jsonSchemaCombineAllOf_meta_filter)(api));
    const outputDir = (0, path_1.resolve)(process.cwd(), options.output);
    // make sure outputDir exists
    (0, fs_1.mkdirSync)(outputDir, { recursive: true });
    // render all templates in the template directory
    for (const file of (0, fs_1.readdirSync)(templateDir)) {
        const template = (0, path_1.join)(templateDir, file);
        const output = (0, path_1.join)(outputDir, file.replace(".njk", ""));
        nunjucks_1.default.render(template, context, (err, res) => {
            console.log(`Writing ${output}`);
            if (err)
                console.error(err);
            (0, fs_1.writeFileSync)(output, res);
        });
    }
}
main();
function extractSchemasFromOpenAPI(schemasIn, api) {
    if (api.paths) {
        for (const path of Object.keys(api.paths)) {
            const pathItem = api.paths[path];
            for (const method of Object.keys(pathItem)) {
                const operation = pathItem[method];
                extractAndAddSchema(operation.requestBody, `${method}_${(0, filter_1.formatPath_filter)(path)}_request_body`);
                const responses = operation.responses;
                for (const key in responses) {
                    extractAndAddSchema(responses[key], `${method}_${(0, filter_1.formatPath_filter)(path)}_response_body_${key}`);
                }
            }
        }
    }
    return schemasIn;
    function extractAndAddSchema(contentObject, name) {
        if (contentObject) {
            if ("content" in contentObject) {
                const content = contentObject.content;
                if (content) {
                    const schema = content["application/json"].schema;
                    addJsonSchema(name, schema, schemasIn);
                }
            }
        }
    }
}
function addJsonSchema(name, schema, schemasIn) {
    const _name = (0, filter_1.lowerSnakeCase_filter)(name);
    if (schema) {
        schema_mapping.push(_name);
        schema_mapping.push(_name + "_write");
        schema_mapping.push(_name + "_read");
        const schemaInput = new quicktype_core_1.JSONSchemaInput(new quicktype_core_1.FetchingJSONSchemaStore());
        const schemaInputWrite = new quicktype_core_1.JSONSchemaInput(new quicktype_core_1.FetchingJSONSchemaStore());
        const schemaInputRead = new quicktype_core_1.JSONSchemaInput(new quicktype_core_1.FetchingJSONSchemaStore());
        schemasIn.addInput(schemaInput);
        schemasIn.addInput(schemaInputWrite);
        schemasIn.addInput(schemaInputRead);
        let schema_string = JSON.stringify(schema);
        if (schema_string.includes('"const":')) {
            schema_string = schema_string.replace(/"const":("[^"]*")/g, `"enum":[$1]`);
        }
        let schema_string_write = schema_string;
        let schema_string_read = schema_string;
        if (schema_string.includes('"readOnly":') ||
            schema_string.includes('"writeOnly":')) {
            schema_string_write = schema_string.replace(/"[^"]*?": {[^{}]*?"readOnly": true[^{}]*?}/gms, ``);
            schema_string_read = schema_string.replace(/"[^"]*?": {[^{}]*?"Only": true[^{}]*?}/gms, ``);
        }
        schemaInput.addSource({
            name: _name,
            schema: schema_string,
        });
        schemaInputWrite.addSource({
            name: _name + "_write",
            schema: schema_string_write,
        });
        schemaInputRead.addSource({
            name: _name + "_read",
            schema: schema_string_read,
        });
    }
}
//# sourceMappingURL=index.js.map